# -*- coding: utf-8 -*-
"""
    :author: luffmims
    :url: http://yum2.cc
    :copyright: © 2025 luffmims <luffmims@hotmaill.com>
    :license: MIT, see LICENSE for more details.
"""

from flask import jsonify, request, render_template
from inventory_check_lark import application, db
from inventory_check_lark.models import init_db, get_access_token, get_table_record, update_record, Inventory
from werkzeug.exceptions import BadRequest

@application.route('/')
def index():
    return render_template('index.html')

@application.route('/readme')
def readme():
    return render_template('readme.html')

@application.route('/init')
def init():
    return render_template('init.html')

@application.route('/import')
def do_init():
    init_db()
    return render_template('after.html',msg="初始化完成")

@application.route('/check', methods=['GET'])
def check():
    inventory = Inventory.query.filter_by(checked=False).first()
    if not inventory:
        return render_template('after.html',msg="未初始化或已盘完")
    return render_template('check.html',inventory=inventory)

@application.route('/get_inventory', methods=['POST'])
def update_inventory():
    try:
        # 获取并验证表单数据
        data = request.get_json()
        inventory_id = data['id']
        amount = data['amount']
        
        # if not inventory_id or not amount:
        #     raise BadRequest('Missing inventory id or amount')

        # 查询并更新库存对象
        inventory = Inventory.query.get(inventory_id)
        if not inventory:
            raise BadRequest('Inventory not found')
        checkedone = inventory.tag
        inventory.amount = amount
        inventory.checked = True
        update_record(inventory.tableid, inventory.recordid, amount)

        # 提交更改
        db.session.commit()

        # 显示成功消息
        # flash(f'{inventory.tag} checked and updated')
        id = inventory_id + 1
        inventory = Inventory.query.get(id)
        if not inventory:
            raise BadRequest('Inventory not found')
        # 返回更新后的库存信息
        return jsonify({
            'id': inventory.id,
            'tag': inventory.tag,
            'amount': inventory.amount,
            'checkedone': checkedone,
            'checkedamount': amount
        })

    except BadRequest as e:
        # 处理错误情况
        # flash(str(e))
        return jsonify({'error': str(e)}), 400

    except Exception as e:
        # 处理其他可能的错误
        db.session.rollback()
        # flash('An error occurred while updating the inventory')
        return jsonify({'error': 'Internal server error'}), 500